// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2011-2013 Mario Guerriero <mario@elementaryos.org>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as published
  by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses/>

  END LICENSE
***/

using Gtk;

namespace Scratch.Widgets {

    public class SplitView : Granite.Widgets.CollapsiblePaned {

        // Widgets
        public Granite.Widgets.Welcome welcome_screen;
        public Scratch.Services.Document? current_document = null;

        public Scratch.Widgets.DocumentView view1 = null;
        public Scratch.Widgets.DocumentView view2 = null;
        public GLib.List<Scratch.Widgets.DocumentView> views = new GLib.List<Scratch.Widgets.DocumentView>();

        // Signals
        public signal void welcome_shown ();
        public signal void welcome_hidden ();
        public signal void document_change (Scratch.Services.Document document);
        public signal void split_view_state_change (bool splitview);

        private weak MainWindow window;

        public SplitView (MainWindow window) {
            base (Gtk.Orientation.HORIZONTAL);
            this.window = window;

            // Welcome screen
            this.welcome_screen = new Granite.Widgets.Welcome (_("No Files Open"),
                                                               _("Open a file to begin editing."));
            this.welcome_screen.valign = Gtk.Align.FILL;
            this.welcome_screen.halign = Gtk.Align.FILL;
            this.welcome_screen.vexpand = true;
            this.welcome_screen.append ("document-new", _("New file"), _("Create a new empty file."));
            this.welcome_screen.append ("document-open", _("Open file"), _("Open a saved file."));
            this.welcome_screen.append ("edit-paste", _("New file from clipboard"), _("Create a new file from the contents of your clipboard."));
            welcome_screen.set_item_visible (2, window.clipboard.wait_is_text_available ());

            this.welcome_screen.activated.connect ((i) => {
                // New file
                if (i == 0)
                    window.main_actions.get_action ("NewTab").activate ();
                // Open
                else if (i == 1)
                    window.main_actions.get_action ("Open").activate ();
                // Clipboard
                else if (i == 2)
                    window.main_actions.get_action ("Clipboard").activate ();
            });

            // Handle Drag-and-drop functionality on source-view
            Gtk.TargetEntry target = {"text/uri-list", 0, 0};
            Gtk.drag_dest_set (this.welcome_screen, Gtk.DestDefaults.ALL, {target}, Gdk.DragAction.COPY);
            this.welcome_screen.drag_data_received.connect ((ctx, x, y, sel, info, time) => {
                var uris = sel.get_uris ();
                if (uris.length > 0) {
                    var view = this.add_view ();

                    for (var i = 0; i < uris.length; i++) {
                        string filename = uris[i];
                        File file = File.new_for_uri (filename);
                        Scratch.Services.Document doc = new Scratch.Services.Document (window.main_actions, file);
                        view.open_document (doc);
                    }

                    Gtk.drag_finish (ctx, true, false, time);
                }
            });
        }

        private Scratch.Widgets.DocumentView crate_new_view () {
            Scratch.Widgets.DocumentView view = new Scratch.Widgets.DocumentView (window);
            view.empty.connect (view_on_empty);
            view.document_change.connect (on_document_changed);
            this.views.append (view);
            this.add (view);
            return view;
        }

        public Scratch.Widgets.DocumentView? add_view () {
            hide_welcome ();

            if (view1 == null) {
                view1 = crate_new_view ();
                this.pack1 (view1, true, true);
                check_actions ();
                return view1;

            } else if (view2 == null) {
                view2 = crate_new_view ();
                this.pack2 (view2, true, true);
                check_actions ();
                return view2;
            }

            check_actions ();

            return get_current_view ();
        }

        private void view_on_empty (Scratch.Widgets.DocumentView empty_view) {
            remove_view (empty_view);
        }

        public void remove_view (Scratch.Widgets.DocumentView? remove_view = null) {
            // If no specific view is required to be removed, just remove the current one
            Scratch.Widgets.DocumentView? view = remove_view;
            if (view == null)
                view = get_unfocused_view ();
            if (view == null) {
                warning ("The is no focused view to remove!");
                return;
            }

            this.remove (view);
            this.views.remove (view);

            merge_views (view);

            view.document_change.disconnect (on_document_changed);

            if(view1 == view) {
                view1.destroy ();
                view1 = null;
            }

            if(view2 == view) {
                view2.destroy ();
                view2 = null;
            }

            check_actions ();

            // Move the focus on the other view
            if (views.nth_data (0) != null) {
                views.nth_data (0).focus ();
            }

            // Show/Hide welcome screen
            if (this.view1 == null && this.view2 == null)
                show_welcome ();

            debug ("View removed succefully");
        }

        private void merge_views (Scratch.Widgets.DocumentView source) {
            debug ("Merge Views");
            Scratch.Widgets.DocumentView target = null;

            if (source == view1 && view2 != null)
                target = view2;
            else if (source == view2 && view1 != null)
                target = view1;

            if (target == null)
                return;

            source.empty.disconnect (view_on_empty);
            
            foreach (Granite.Widgets.Tab? tab in source.notebook.tabs) {
                if (tab == null)
                    continue;
                // Check if the tab is not a temporery and empty file
                if (!(tab as Scratch.Services.Document).delete_temporary_file ()) {
                    source.notebook.remove_tab (tab);
                    target.notebook.insert_tab (tab, target.notebook.n_tabs);
                }
            }
        }

        public Scratch.Widgets.DocumentView? get_current_view () {
            Scratch.Widgets.DocumentView? return_value = get_focus_child () as Scratch.Widgets.DocumentView;

            if (return_value != null)
                return return_value;

            return view1 ?? view2;
        }

        private Scratch.Widgets.DocumentView? get_unfocused_view () {
            Scratch.Widgets.DocumentView? current_view = get_current_view();
            debug ("Current view is null: %s", (current_view == null).to_string ());
            if (current_view == view1 && view2 != null) 
                return view2;
            else if (current_view == view2 && view1 != null)
                return view1;

            return current_view;
        }

        public bool is_empty () {
            return (views.length () == 0);
        }

        // Show welcome screen
        public void show_welcome () {
            this.pack1 (welcome_screen, true, true);
            this.welcome_screen.show_all ();
            welcome_shown ();
            debug ("WelcomeScreen shown succefully");
        }

        // Hide welcome screen
        public void hide_welcome () {
            if (this.welcome_screen.get_parent () == this) {
                this.remove (welcome_screen);
                welcome_hidden ();
                debug ("WelcomeScreen hidden succefully");
            }
        }

        // Detect the last focused Document throw a signal
        private void on_document_changed (Scratch.Services.Document? document) {
            current_document = document;

            if (current_document != null)
                document_change (current_document);
            
        }

        // Check the possibility to add or not a new view
        private void check_actions (){
            split_view_state_change (view1 != null && view2 != null);
        }
    }
}

